#include <avr/io.h>
#include "vbus.h"
#include "buff.h"
#include "config.h"
#include "timer.h"
#include "ntp.h"


/* Variablen fr die Protokollanalyse deklarieren
 * Geht sicherlich noch kleiner 
 * Man knnte z.B. vbus_head und vbus_frame verschmelzen, 
 * wird dann aber unbersichtlicher
 */
volatile t_vbus_head vbus_head;
t_vbus10_frame vbus_frame;

//Prfsumme eines Pakets berechnen
 uint8_t VBus_CalcCrc(const unsigned char *Buffer, uint8_t Offset, uint8_t len) {
	uint8_t crc = VBUS_CRC_MAGIC;
	while(len) {
		len--;
		crc -= Buffer[Offset++];
		crc &= VBUS_CRC_MAGIC;
	}
	return crc;
}

void VBus_ProcessChar(uint8_t c) {
	if(vbus_head.update == 0) return;
	if(c == VBUS_SYNC) {
		//Wir haben Sync -> alle folgenden Daten werden decodiert
		vbus_head.sync = 1;
		//Offset im Head setzen
		vbus_head.offset = 1;
		//Das nchste empfangene Frame hat den Index 0
		vbus_head.currframe = 0;
		/* Aktuelle Daten als ungltig markieren
		 * Geht theoretisch auch spter, an dieser Stelle knnen aber CPU-Zyklen gespart werden ;)
		 */
		vbus_head.valid = 0;
		
		VBUS_DEBUG_S("Sync ");

		//Wenn das Sync-Wort empfangen wurde, gibt es fr das aktuelle Byte nichts mehr zu tun
		return;
	}
	
	//Daten nur erfassen/verarbeiten, wenn vorher ein sync erkannt wurde
	if(vbus_head.sync) {

		
		//Paket-Head in die vorgesehene Variable schreiben
		if(vbus_head.offset < VBUS_HEADER_LENGTH) {
			uint8_t *headptr = (uint8_t *)&vbus_head;
			headptr += vbus_head.offset-1;
			*headptr = c;
		}
		
		if(vbus_head.offset == (VBUS_HEADER_LENGTH - 1)) {
			
			/* Nach dem 10. Byte (inkl. Startwort) ist der Header komplett
			 * Jetzt kann er geprft und ausgewertet werden
			 */
			VBUS_DEBUG_BREAK();
			VBUS_DEBUG_S("D:");
			VBUS_DEBUG_DH(vbus_head.dest);
			VBUS_DEBUG_S(" S:");
			VBUS_DEBUG_DH(vbus_head.source);
			VBUS_DEBUG_S(" V:");
			VBUS_DEBUG_C((vbus_head.version >> 4) + 48);
			VBUS_DEBUG_S(" C:");
			VBUS_DEBUG_DH(vbus_head.command);
			VBUS_DEBUG_S(" F:");
			VBUS_DEBUG_I(vbus_head.frames);
			VBUS_DEBUG_S(" X:");

			/* Checksum berprfen
			 * Ist sie inkorrekt knnen wir mit dem Head und somit auch 
			 * mit allen folgenden Daten nichts anfangen
			 */
			if(VBus_CalcCrc((uint8_t *)&vbus_head, 0, 8) != vbus_head.checksum) {
				//Prfsumme ist ungltig, sync zurcksetzen und beenden
				VBUS_DEBUG_S("nok");
				vbus_head.sync = 0; 
				return;
			}
			VBUS_DEBUG_S("ok");

			//Protokollversion prfen, aktuell wird nur 1.0 untersttzt
			if(vbus_head.version != VBUS_PROTO_10) {
				//Protokollversion ist ungleich 1.0, sync zurcksetzen und beenden
				vbus_head.sync = 0;
				VBUS_DEBUG_BREAK();
				VBUS_DEBUG_S("!V1.0->dropped ");
				return;
			}

			//Prfen, ob das Paket richtig adressiert ist
			if(vbus_head.source != VBUS_SRC || vbus_head.dest != VBUS_DST) {
				//Paket ist nicht fr "mich", ignorieren
				VBUS_DEBUG_S("Wrong addr->dropped ");
				vbus_head.sync = 0;
				return;
			}

			/* Hier knnten wir noch berprfen, ob die Frame-Lnge und der Befehl stimmt
			 * Ist aber nicht ntig, da in diesem Fall nur eine Paketart mit entsprechenden
			 * Adressen versendet werden
			 */
		}

		//Ab Offset 10 (VBUS_HEADER_LENGTH) sind wir im Datenbereich
		if(vbus_head.offset >= VBUS_HEADER_LENGTH) {
			/* Offset im Paket berechnen
			 * kann man sicherlich auch komplett per Pointer machen, 
			 * ist mir an dieser Stelle aber ein bisschen zu unheimlich ;)
			 * 10 (VBUS_HEADER_LENGTH) ist der Offset (VBUS-Header), 6 (VBUS_10_FRAME_LENGTH) die Framelnge
			 */
			uint8_t packetoffset = (vbus_head.offset - VBUS_HEADER_LENGTH) % VBUS_10_FRAME_LENGTH; 
			
			//Wert an der aktuellen Stelle in die Paketvariable schreiben
			volatile unsigned char *pos = (volatile unsigned char *)&vbus_frame;
			pos += packetoffset;
			*pos = c;

			//Frame ist fertig. Jetzt schauen, ob es gltig ist.
			//das -1 ergibt sich wegen der Basis 0!
			if(packetoffset == (VBUS_10_FRAME_LENGTH-1)) {
				VBUS_DEBUG_S(" F");
				VBUS_DEBUG_I(vbus_head.currframe);

				//Pointer aufs Ausgabestruct ermitteln
				volatile unsigned char *outptr = (volatile unsigned char *)vbus_head.od;
				outptr += vbus_head.currframe * VBUS_10_FRAME_PAYLOAD; //Frames sind 4 Byte lang, Position des Pointers setzen
				
				uint8_t i = VBUS_10_FRAME_PAYLOAD;
				if(VBus_CalcCrc((uint8_t *)&vbus_frame, 0, (VBUS_10_FRAME_PAYLOAD + 1)) != vbus_frame.checksum) {
					//Speicher beschreiben, wenn das Frame ungltig ist
					while(i--) {
						*outptr = VBUS_INVALID_FRAME;
						outptr++;
					}
					VBUS_DEBUG_S("nok");
				} else {
					//Prfsumme war korrekt -> Septett holen
					uint8_t sept = vbus_frame.septett;
					uint8_t *inptr = (uint8_t *)&vbus_frame;
					
					VBUS_DEBUG_S("ok");

					while(i--) {
						*outptr = *inptr; //Wert aus dem Frame in die Ausgabe bertragen
						//if(sept & 1) *outptr |= 0x80; //Septett hinzufgen
						*outptr |= (sept & 1) << 7; //ein stckchen kleiner als die obere Zeile ;-)
						
						//Wer den Debug ein bisschen detaillierter haben will:
						/*VBUS_DEBUG_H(*outptr);
						VBUS_DEBUG_C(' ');*/

						sept >>= 1; //Septett nach rechts schieben (nchstes Bit an erste Stelle)
						outptr++;
						inptr++;
					}
				}
				vbus_head.currframe++;
				if(vbus_head.currframe == VBUS_FRAME_COUNT) {
					vbus_head.valid = 1;
					vbus_head.update = 0;
					VBUS_DEBUG_S(" AFR");
					vbus_head.od->timestamp = GetUnixTimestamp(time);
				}
			}
		}
		vbus_head.offset++;
	}
}

void VBus_RequestData(t_vbus_outdata *dst) {
	vbus_head.sync = 0;
	vbus_head.od = dst;
	vbus_head.update = 1;
}

uint8_t VBus_ToJson(t_vbus_outdata *data, char *ptr, uint8_t index) {
	uint8_t pos = 0;
	uint8_t i;

	switch(index) {
		case 0:
			pos = buff_str_p("\"temp\":[", ptr, pos, BUFFER_LEN); //"temp":[

			//Temperaturen
			for(i = 0; i < 12; i++) {
				pos = buff_solar_temp(data->temp[i], buff, pos, BUFFER_LEN);
				pos = buff_chr(buff, ',', pos, BUFFER_LEN);
			}
			pos--;//letztes Komma abschneiden
			pos = buff_chr(buff, 0, pos, BUFFER_LEN);
			return pos;
		case 1:
			//Einstrahlung
			pos = buff_str_p("],\"radiation\":", ptr, pos, BUFFER_LEN); //],"radiation":
			pos = buff_int(data->irradiation, ptr, pos, BUFFER_LEN, 10);

			//Impulse
			pos = buff_str_p(",\"impulse\":[", ptr, pos, BUFFER_LEN); //,"impulse":[
			
			for(i = 0; i < 2; i++) {
				pos = buff_int(data->impulse[i], ptr, pos, BUFFER_LEN, 10);
				pos = buff_str_p(",", ptr, pos, BUFFER_LEN); //"],
			}
			pos--;
			
			//Sensoren
			pos = buff_str_p("],\"sensor\":{\"broke\":", ptr, pos, BUFFER_LEN); //],"sensor":{"broke":
			pos = buff_int(data->sensorbroke, ptr, pos, BUFFER_LEN, 10);
			pos = buff_chr(buff, 0, pos, BUFFER_LEN);

			return pos;
		case 2:
			
			pos = buff_str_p(",\"short\":", ptr, pos, BUFFER_LEN); //,"short":
			pos = buff_int(data->sensorshort, ptr, pos, BUFFER_LEN, 10);
			pos = buff_str_p(",\"usage\":", ptr, pos, BUFFER_LEN); //,"usage":
			pos = buff_int(data->sensorusage, ptr, pos, BUFFER_LEN, 10);


			//Pumpen
			pos = buff_str_p("},\"pump\":[", ptr, pos, BUFFER_LEN); //},"pump":[
			for(i = 0; i < 10; i++) {
				pos = buff_int(data->pump[i] / 10, ptr, pos, BUFFER_LEN, 10);
				pos = buff_chr(buff, ',', pos, BUFFER_LEN);
			}
			pos--;//letztes Komma abschneiden


			pos = buff_chr(buff, 0, pos, BUFFER_LEN);
			return pos;
		case 3:

			//Relais
			pos = buff_str_p("],\"relais\":", ptr, pos, BUFFER_LEN); //],"relais":
			pos = buff_int(data->relaisusage, ptr, pos, BUFFER_LEN, 10);

			//Fehler
			pos = buff_str_p(",\"errors\":", ptr, pos, BUFFER_LEN); //,"errors":
			pos = buff_int(data->errors, ptr, pos, BUFFER_LEN, 10);

			//Warnungen
			pos = buff_str_p(",\"warnings\":", ptr, pos, BUFFER_LEN); //,"warnings":
			pos = buff_int(data->warnings, ptr, pos, BUFFER_LEN, 10);

			//Zeit
			pos = buff_str_p(",\"time\":", ptr, pos, BUFFER_LEN); //,"time":
			pos = buff_int(data->time, ptr, pos, BUFFER_LEN, 10);
			pos = buff_str_p(",\"timestamp\":", ptr, pos, BUFFER_LEN); //,"timestamp":
			pos = buff_int(data->timestamp, ptr, pos, BUFFER_LEN, 10);

			pos = buff_chr(buff, 0, pos, BUFFER_LEN);
			return pos;	
	}
	return 0;
}

